@@ -22,6 +22,7 @@ import errno, os, os.path, re, pty, signal, tempfile
 
 from gettext               import gettext as _
 from gnome_lirc_properties import config, lirc
+from gnome_lirc_properties import shellquote as ShellQuote
 from StringIO              import StringIO
 
 # Modern flavors of dbus bindings have that symbol in dbus.lowlevel,
@@ -64,18 +65,15 @@ class PolicyKitService(dbus.service.Object):
             if sender:
                 kit = dbus.SystemBus().get_object('org.freedesktop.PolicyKit', '/')
                 kit = dbus.Interface(kit, 'org.freedesktop.PolicyKit')
-                pid = dbus.UInt32(os.getpid())
-
-                granted = kit.IsProcessAuthorized(action, pid, False)
-                logging.info('process authorization: %r', granted)
-
-                if 'no' == granted:
-                    raise AccessDeniedException('Process not authorized by PolicyKit')
 
+                # Note that we don't use IsProcessAuthorized because we have
+                # no ways to get the PID of the front-end, so we're left
+                # with checking that its bus name is authorised instead
+		# See http://bugzilla.gnome.org/show_bug.cgi?id=540912
                 granted = kit.IsSystemBusNameAuthorized(action, sender, False)
-                logging.info('authorizatoin of system bus name: %r', granted)
+                logging.info('authorization of system bus name \'%s\': %r', sender, granted)
 
-                if 'no' == granted:
+                if 'yes' != granted:
                     raise AccessDeniedException('Session not authorized by PolicyKit')
 
         except AccessDeniedException:
@@ -100,6 +98,7 @@ class ExternalToolDriver(PolicyKitService):
         self.__line_buffer = ''
         self.__pid = -1
         self.__fd = -1
+        self._hup_expected = False
 
     def _spawn_external_tool(self):
         '''Launches the external tool backing this service.'''
@@ -162,6 +161,7 @@ class ExternalToolDriver(PolicyKitService):
         self.__pid, self.__fd = self._spawn_external_tool()
 
         if -1 != self.__fd:
+            self._hup_expected = False
             gobject.io_add_watch(self.__fd,
                                  gobject.IO_IN | gobject.IO_HUP,
                                  self.__io_handler)
@@ -246,6 +246,7 @@ class IrRecordDriver(ExternalToolDriver):
     _token_next_key         = 'Please enter the name for the next button'
     _token_wait_toggle_mask = 'If you can\'t see any dots appear'
     _token_finished         = 'Successfully written config file.'
+    _token_almost_finished  = 'Creating config file in raw mode.' # Shown when _token_finished should not be expected.
 
     # Last instance index for automatic object path creation:
     __last_instance = 0
@@ -264,11 +265,12 @@ class IrRecordDriver(ExternalToolDriver):
         self._filename = filename
 
         if device:
-            self._cmdargs.append('--device=%s' % device)
+            self._cmdargs.append('--device=%s' % ShellQuote.shellquote(device))
         if filename:
             self._cmdargs.append(filename)
 
         super(IrRecordDriver, self).__init__(connection, path)
+        logging.info("__init__.IrRecordDriver(): irrecord cmdargs=%s", self._cmdargs)
 
     def _on_run_external_tool(self):
         '''Enters to the working directory and executes irrecord.'''
@@ -296,7 +298,15 @@ class IrRecordDriver(ExternalToolDriver):
     def _on_hangup(self):
         '''Shutdown the driver, when irrecord terminates unexpectedly.'''
 
-        if list(self.locations):
+        if(self._hup_expected):
+            # This was expected. It's a success:
+            filename = os.path.join(self._workdir, self._filename)
+            configuration = open(filename).read()
+            self.ReportSuccess(configuration)
+            self.Release()
+
+        if list(self.locations): #self.locations is in the dbus base class. This check prevents a shutdown when the dbus service should be kept alive.
+            # irrecord stopped when we were expecting more interaction:
             self.ReportFailure(_('Custom remote control configuration aborted unexpectedly.'))
             self.Release()
 
@@ -321,6 +331,7 @@ class DetectParametersDriver(IrRecordDriver):
     def __init__(self, connection, driver, device):
         super(DetectParametersDriver, self).__init__(connection, driver, device)
         self.__report_progress = False
+        self._hup_expected = False
 
     def _prepare_workdir(self):
         '''Removes the configuration file from working directory when needed.'''
@@ -384,6 +395,15 @@ class DetectParametersDriver(IrRecordDriver):
             self.Release()
             return
 
+        if line.startswith(self._token_almost_finished):
+            # For some remotes (such as the XBox replacement remote, 
+            # with the StreamZap receiver, there is no additional output after 
+            # pressing Enter to stop the key-code learning, 
+            # so _token_finished is never seen,
+            # but in that case we see this message instead beforehand.
+            self._hup_expected = True
+            return
+
     def _on_next_chunk(self, chunk):
         '''
         Identifies progress indications in the next chunk of irrecord output,
@@ -454,20 +474,7 @@ class LearnKeyCodeDriver(IrRecordDriver):
         return None
 
     def _spawn_external_tool(self):
-        '''Runs irrecord with --resume switch when supported.'''
-        irrecord = os.popen('%s --help' % config.LIRC_IRRECORD)
-
-        if -1 == irrecord.read().find('--resume'):
-            logging.warning('irrecord doesn\'t have --resume switch')
-
-            self.ReportFailure(_(
-                'The installed lirc does not support key code learning ' +
-                'because its irrecord command does not have the --resume ' +
-                'option. Please file a bug against your Linux distribution.'))
-
-            return 0, -1
-
-        self._cmdargs.insert(1, '--resume')
+        '''Runs irrecord.'''
         return super(LearnKeyCodeDriver, self)._spawn_external_tool()
 
 class BackendService(PolicyKitService):
@@ -573,7 +580,7 @@ class BackendService(PolicyKitService):
 
             if value is not None:
                 logging.info('- writing %s"%s"', match.group(0), value)
-                print >> output, ('%s"%s"' % (match.group(0), value))
+                print >> output, ('%s"%s"' % (match.group(0), ShellQuote.shellquote(value)))
                 continue
 
             # Identify directives starting with RECEIVER_ and replacing their values with ours.
@@ -583,7 +590,7 @@ class BackendService(PolicyKitService):
 
             if value is not None:
                 logging.info('- writing %s"%s"', match.group(0), value)
-                print >> output, ('%s"%s"' % (match.group(0), value))
+                print >> output, ('%s"%s"' % (match.group(0), ShellQuote.shellquote(value)))
                 continue
 
             # Deal with the START_LIRCD line:
@@ -598,7 +605,7 @@ class BackendService(PolicyKitService):
                 value = (start_lircd is None) and 'true' or start_lircd
                 start_lircd = None
 
-                print >> output, (match.group(0) + value)
+                print >> output, (match.group(0) + ShellQuote.shellquote(value))
                 continue
 
             output.write(line)
@@ -609,12 +616,12 @@ class BackendService(PolicyKitService):
         if remote_values:
             print >> output, '\n# Remote settings required by gnome-lirc-properties'
         for key, value in remote_values.items():
-            print >> output, ('REMOTE_%s="%s"' % (key, value))
+            print >> output, ('REMOTE_%s="%s"' % (key, ShellQuote.shellquote(value)))
 
         if receiver_values:
             print >> output, '\n# Receiver settings required by gnome-lirc-properties'
         for key, value in receiver_values.items():
-            print >> output, ('RECEIVER_%s="%s"' % (key, value))
+            print >> output, ('RECEIVER_%s="%s"' % (key, ShellQuote.shellquote(value)))
 
         if start_lircd is not None:
             print >> output, '\n# Daemon settings required by gnome-lirc-properties'
@@ -687,10 +694,9 @@ class BackendService(PolicyKitService):
             self._write_hardware_configuration(remote_values=values)
 
         # Write remote configuration:
-        filename, include_needed = lirc.find_remote_config()
+        filename = lirc.find_remote_config()
 
-        if include_needed:
-            self.__write_include_statement(filename)
+        self.__write_include_statement(filename)
 
         print contents
 
@@ -718,8 +724,8 @@ class BackendService(PolicyKitService):
             contents = ''
 
         # find existing include statement:
-        include_statement = 'include %s\n' % redirect
-        pattern = r'^\s*(#.*)?include\s+%s\s*$' % re.escape(redirect)
+        include_statement = 'include <%s>\n' % redirect
+        pattern = r'^\s*(#.*)?include\s+<%s>\s*$' % re.escape(redirect)
         pattern = re.compile(pattern, re.MULTILINE)
         match   = pattern.search(contents)
 
@@ -732,7 +738,7 @@ class BackendService(PolicyKitService):
                 contents += '#\n'
                 contents += '# Feel free to add any custom remotes to the configuration\n'
                 contents += '# via additional include directives or below the existing\n'
-                contents += '# Ubuntu include directives from your selected remote and/or\n'
+                contents += '# include directives from your selected remote and/or\n'
                 contents += '# transmitter.\n'
                 contents += '#\n'
 
@@ -757,10 +763,15 @@ class BackendService(PolicyKitService):
     def ManageLircDaemon(self, action, sender=None):
         '''Starts the LIRC daemon.'''
 
+        permitted_actions = [ 'enable', 'disable', 'stop', 'start', 'restart' ]
+
         self._check_permission(sender)
 
         print 'Managing lircd: %s...' % action
 
+	if action not in permitted_actions:
+	    raise AccessDeniedException
+
         if 'enable' == action:
             self._write_hardware_configuration(start_lircd=True)
 
